#ifndef CAMERA_H
#define CAMERA_H

//    This file is part of Motion Watch.

//    Motion Watch is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    Motion Watch is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "common.h"
#include "logger.h"
#include "web.h"
#include "mo_detect.h"

class Loop : public QObject
{
    Q_OBJECT

protected:

    shared_t *shared;
    int       heartBeat;

private:

    void loop();

public:

    explicit Loop(shared_t *shared, QObject *parent = nullptr);

    virtual bool exec();
};

class Camera : public QObject
{
    Q_OBJECT

private:

    shared_t shared;

public:

    explicit Camera(QObject *parent = nullptr);

    bool start(const QStringList &args);
};

class RecLoop : public Loop
{
    Q_OBJECT

private:

    QProcess   proc;
    QByteArray streamMD5;
    bool       once;

    void updateCmd();
    void reset();

public:

    explicit RecLoop(shared_t *shared, QObject *parent = nullptr);

    bool exec();
};

class Upkeep : public Loop
{
    Q_OBJECT

public:

    explicit Upkeep(shared_t *shared, QObject *parent = nullptr);

    bool exec();
};

class EventLoop : public Loop
{
    Q_OBJECT

private:

    bool wrOutVod(const evt_t &event);

public:

    explicit EventLoop(shared_t *shared, QObject *parent = nullptr);

    bool exec();
};

class DetectLoop : public Loop
{
    Q_OBJECT

private:

    int     pcId;
    int     evId;
    QString prevTs;

    void resetTimers();
    void timerEvent(QTimerEvent *event);

public:

    explicit DetectLoop(shared_t *shared, QObject *parent = nullptr);

    bool exec();
};

#endif // CAMERA_H
